# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-06 15:54
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('jsonattrs', '0004_convert_multilingual_fields'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attribute',
            name='choice_labels',
        ),
        migrations.RemoveField(
            model_name='attribute',
            name='long_name',
        ),
    ]
