# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-06 15:54
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.migrations.operations.special


class Migration(migrations.Migration):

    dependencies = [
        ('jsonattrs', '0002_attribute_choice_labels'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='choice_labels_xlat',
            field=django.contrib.postgres.fields.jsonb.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='attribute',
            name='long_name_xlat',
            field=django.contrib.postgres.fields.jsonb.JSONField(default={}),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='schema',
            name='default_language',
            field=models.CharField(blank=True, max_length=3),
        ),
    ]
